import curses
import random
from .snake import Snake

class Game:
    def __init__(self, stdscr):
        self.stdscr = stdscr
        curses.curs_set(0)       
        self.stdscr.nodelay(True)
        self.base_time = 200
        self.stdscr.timeout(self.base_time)  
        self.sr, self.sc = self.stdscr.getmaxyx()
        start_x = self.sr // 2
        start_y = self.sc // 2
        
        self.snake = Snake(start_x, start_y, self.sr, self.sc)
        self.food = self._place_food()
        self.score = 0
        self.speed_level = 1
    
    def _place_food(self):
        while True:
            fx = random.randint(0, self.sr - 1)
            fy = random.randint(0, self.sc - 1)
            if (fx, fy) not in self.snake.positions:
                return (fx, fy)
    def _update_speed(self):
        if self.score % 5 == 0 and self.score > 0:
            self.speed_level += 1
            new_delay = max(self.base_time - (self.speed_level * 20), 50)
            self.stdscr.timeout(new_delay)
    def _exit(self):
        self.stdscr.clear()
        msg = "Game Over! Press any key to exit."
        self.stdscr.addstr(self.sr // 2, self.sc // 2 - len(msg) // 2, msg)
        self.stdscr.refresh()
        self.stdscr.nodelay(False) 
        self.stdscr.getch()
    def run(self):
        """
        The main game loop.
        """
        while True:
            # Get user input
            try:
                key = self.stdscr.getch()
            except:
                key = -1     
            if key == curses.KEY_UP:
                self.snake.change_direction('UP')
            elif key == curses.KEY_DOWN:
                self.snake.change_direction('DOWN')
            elif key == curses.KEY_LEFT:
                self.snake.change_direction('LEFT')
            elif key == curses.KEY_RIGHT:
                self.snake.change_direction('RIGHT')
            elif key == ord('q'):
                # Press 'q' to quit
                break
            
            # Move the snake
            try:
                self.snake.move()
            except ValueError:
                break
            
            # Check if snake ate food
            if (self.snake.head.x, self.snake.head.y) == self.food:
                # Grow the snake
                try:
                    self.snake.grow()
                except ValueError:
                    break
                self.score +=1
                self._update_speed()
                self.food = self._place_food()
            # draw stuffs 
            self.stdscr.clear()
            self.stdscr.addch(self.food[0], self.food[1], curses.ACS_PI)
            node = self.snake.head
            while node:
                self.stdscr.addch(node.x, node.y, '#')
                node = node.next
            status = f"Score: {self.score}"
            self.stdscr.addstr(0, 0, status[:self.sc-1]) 
            self.stdscr.refresh()

        self._exit()



class Node:
    def __init__(self, x, y):
        self.x = x
        self.y = y
        self.next = None
        self.prev = None

class Snake:
    def __init__(self, first_x, first_y, x_max, y_max):
        node = Node(first_x, first_y)
        self.head = node
        self.tail = node
        self.direction = 'RIGHT'
        self.positions = {(first_x, first_y)}
        self.x_max = x_max
        self.y_max = y_max
    
    def _wrap(self, x, y):
        # Wrap row
        if x < 0:
            x = self.x_max - 1
        elif x >= self.x_max:
            x = 0
        # Wrap column
        if y < 0:
            y = self.y_max - 1
        elif y >= self.y_max:
            y = 0
        return x, y
    
    def move(self):
        x, y = self.head.x, self.head.y
        if self.direction == 'UP':
            x-=1
        if self.direction == 'DOWN':
            x+=1
        if self.direction == 'RIGHT':
            y+=1
        if self.direction == 'LEFT':
            y-=1   
        
        x, y = self._wrap(x, y)
        x_tail, y_tail = self.tail.x, self.tail.y
        self.positions.remove((x_tail, y_tail))
        if self.tail == self.head:  # Snake has only one node
            self.head = None
            self.tail = None
        else:
            if self.tail.prev:
                self.tail = self.tail.prev 
                self.tail.next = None
            else:
                self.tail = None
        
        self.check_collision(x,y)


        new_head = Node(x,y)
        new_head.next = self.head
        if self.head:
            self.head.prev = new_head
        self.head = new_head
        if not self.tail:
            self.tail = self.head
        self.positions.add((x,y))

    def grow(self):
        x, y = self.head.x, self.head.y
        if self.direction == 'UP':
            x -= 1
        elif self.direction == 'DOWN':
            x += 1
        elif self.direction == 'LEFT':
            y -= 1
        elif self.direction == 'RIGHT':
            y += 1
        # Note that my tail is my head
        new_head = Node(x, y)
        new_head.next = self.head
        if self.head:
            self.head.prev = new_head
        self.head = new_head
        self.positions.add((x,y))

    def change_direction(self, new_direction):
        opposites = {'UP':'DOWN', 'DOWN': 'UP','LEFT': 'RIGHT','RIGHT': 'LEFT'}
        if new_direction != opposites.get(self.direction):
            self.direction = new_direction
    
    def check_collision(self, x,y):
        if(x,y) in self.positions:
            raise ValueError("Collision game over")
        return False



from setuptools import setup, find_packages

setup(
    name='snake-terminal',
    version='1.0.0',
    description="Snake game in your terminal",
    author="Alabi Busayo",
    author_email="busayoalabi1234@gmail.com",
    url="https://github.com/busayojee/snake-terminal",
    classifiers=[
        "Environment :: Console :: Curses",
        "Natural Language :: English",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: POSIX",
        "Operating System :: Unix",
        "Programming Language :: Python :: 3",
        "Topic :: Utilities",
    ],
    packages=find_packages(),
    install_requires=[
       "curses-menu>=0.7.1",
       "click>=8.1.7"
    ],
    entry_points={
        'console_scripts': [
            'snake-terminal = snaketerm.main:main',
        ],
    },
)


import curses
from .game import Game
import click
from . import VERSION_STRING

def play(stdscr):
    game = Game(stdscr)
    game.run()
def print_version(ctx, param, value):
    if not value or ctx.resilient_parsing:
        return
    click.echo(VERSION_STRING)
    ctx.exit()
@click.command()
@click.option("-v", "--version", is_flag=True, callback=print_version,expose_value=False, is_eager=True, help="Show version and exit")
def main():
    curses.wrapper(play)
    
# if __name__ == '__main__':
#     main()
    
